/**
 *    @file  clusterlst.c
 *   @brief  Self initialized cluster records list (non thread-safe)
 *
 *  <+DETAILED+>
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  25/10/2024
 *      Revision:  none
 * Last modified:  2024-11-19 18:42
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "clusterlst.h"
#include "sclist.h"

#include <stdlib.h>
#include <string.h>

static sclist_t* clusterlist=NULL;
static sclistrecord_t* clusterlist_cursor=NULL;

static void clusterlist_add_postinit (cluster_t* cluster);
static cluster_t* clusterlist_find_postinit (const char* host);
static cluster_t* clusterlist_first_postinit();
static cluster_t* clusterlist_next_postinit();
static cluster_t* clusterlist_get_postinit();

static void clusterlist_init() {
    /* Initialize the structure */
    clusterlist = sclist_new();
    clusterlist_cursor = NULL;

    /* From now, the application can directly use the real functions */
    clusterlist_add = clusterlist_add_postinit;
    clusterlist_find = clusterlist_find_postinit;
    clusterlist_first = clusterlist_first_postinit;
    clusterlist_next = clusterlist_next_postinit;
    clusterlist_get = clusterlist_get_postinit;
}

static void clusterlist_add_postinit (cluster_t* cluster) {
    sclist_addrecord(clusterlist,cluster);
}
static void clusterlist_add_preinit (cluster_t* cluster) {
    clusterlist_init();
    clusterlist_add(cluster);
}
void (*clusterlist_add)(cluster_t* cluster) = clusterlist_add_preinit;


static cluster_t* clusterlist_find_postinit(const char* host) {
    cluster_t* cluster=NULL;
    clusterlist_cursor = sclist_firstrecord(clusterlist);
    while (clusterlist_cursor) {
        cluster = sclist_getvalue(clusterlist_cursor);
        if (
            ((host==NULL)&&(cluster->host==NULL))||
            ((host!=NULL)&&(cluster->host!=NULL)&&(0==strcmp(host,cluster->host))))
            break;
        clusterlist_cursor = sclist_nextrecord(clusterlist_cursor);
    };
    return (clusterlist_cursor?cluster:NULL);
}
static cluster_t* clusterlist_find_preinit(const char* host) {
    clusterlist_init();
    return clusterlist_find(host);
}
cluster_t* (*clusterlist_find)(const char* host) = clusterlist_find_preinit;


static cluster_t* clusterlist_first_postinit() {
    clusterlist_cursor = sclist_firstrecord(clusterlist);
    return (clusterlist_cursor?sclist_getvalue(clusterlist_cursor):NULL);
}
static cluster_t* clusterlist_first_preinit() {
    clusterlist_init();
    return clusterlist_first();
}
cluster_t* (*clusterlist_first)() = clusterlist_first_preinit;


static cluster_t* clusterlist_next_postinit() {
    clusterlist_cursor = (clusterlist_cursor?sclist_nextrecord(clusterlist_cursor):NULL);
    return (clusterlist_cursor?sclist_getvalue(clusterlist_cursor):NULL);
}
static cluster_t* clusterlist_next_preinit() {
    clusterlist_init();
    return clusterlist_next();
}
cluster_t* (*clusterlist_next)() = clusterlist_next_preinit;


static cluster_t* clusterlist_get_postinit() {
    return (clusterlist_cursor?sclist_getvalue(clusterlist_cursor):NULL);
}
static cluster_t* clusterlist_get_preinit() {
    clusterlist_init();
    return clusterlist_get();
}
cluster_t* (*clusterlist_get)() = clusterlist_get_preinit;

/* vim: set tw=80: */
