/**   @file  suite_sample.c
 *   @brief  Check testsuite template
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  20/03/2022
 *      Revision:  none
 * Last modified:  2024-11-11 21:33
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <check.h>
#include <unistd.h>                             /* pid_t getpid() */
#include <sys/stat.h>                           /* mkdir(), chdir(), mode_t */
#include <stdio.h>                              /* fprintf */
#include <stdlib.h>                             /* abort() */
#include "checktools.inc"

/** Dummy test to trigger fixtures
 *
 * This test is defined and added to all testcases in order to execute at
 * least one test test in each testcase of the testsuite, trigger the
 * fixtures and have a decent coverage report.
 */
START_TEST(sample_test) {
    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    abort();
}
END_TEST

/************************************************************************/
/* Begining of test (potentially in child) */
void sample_checked_uninitialized_setup() {
    signals_catch();
    forktest_gprofdir();
}

/* End of test (potentially in child) */
void sample_checked_uninitialized_teardown() {
    signals_release();
}
/************************************************************************/
void sample_unchecked_common_setup() {
    forktest_init();
}
void sample_unchecked_common_teardown() {
}
/************************************************************************/
Suite* sample_suite() {
    Suite *s;
    TCase *tc;

    s = suite_create("SampleSuite");

    tc = tcase_create("SampleCase");
    tcase_set_tags(tc,"SampleTag SampleTag2");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, sample_checked_uninitialized_setup, sample_checked_uninitialized_teardown);
    tcase_add_unchecked_fixture(tc, sample_unchecked_common_setup, sample_unchecked_common_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test_raise_signal(tc, sample_test,6);

    return s;
}
/* vim: set tw=80: */
